package kr.co.vividnext.sodalive.admin.calculate

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@PreAuthorize("hasRole('ADMIN')")
@RequestMapping("/admin/calculate")
class AdminCalculateController(private val service: AdminCalculateService) {
    @GetMapping("/live")
    fun getCalculateLive(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String
    ) = ApiResponse.ok(service.getCalculateLive(startDateStr, endDateStr))

    @GetMapping("/content-list")
    fun getCalculateContentList(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String
    ) = ApiResponse.ok(service.getCalculateContentList(startDateStr, endDateStr))

    @GetMapping("/cumulative-sales-by-content")
    fun getCumulativeSalesByContent(pageable: Pageable) = ApiResponse.ok(
        service.getCumulativeSalesByContent(pageable.offset, pageable.pageSize.toLong())
    )

    @GetMapping("/content-donation-list")
    fun getCalculateContentDonationList(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String
    ) = ApiResponse.ok(service.getCalculateContentDonationList(startDateStr, endDateStr))

    @GetMapping("/community-post")
    fun getCalculateCommunityPost(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String,
        pageable: Pageable
    ) = ApiResponse.ok(
        service.getCalculateCommunityPost(
            startDateStr,
            endDateStr,
            pageable.offset,
            pageable.pageSize.toLong()
        )
    )
}
