package kr.co.vividnext.sodalive.admin.calculate

import kr.co.vividnext.sodalive.creator.admin.calculate.GetCreatorCalculateCommunityPostResponse
import kr.co.vividnext.sodalive.extensions.convertLocalDateTime
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminCalculateService(private val repository: AdminCalculateQueryRepository) {
    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["default"],
        key = "'calculateLive:' + " + "#startDateStr + ':' + #endDateStr"
    )
    fun getCalculateLive(startDateStr: String, endDateStr: String): List<GetCalculateLiveResponse> {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        return repository
            .getCalculateLive(startDate, endDate)
            .map { it.toGetCalculateLiveResponse() }
    }

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["cache_ttl_3_hours"],
        key = "'calculateContent:' + " + "#startDateStr + ':' + #endDateStr"
    )
    fun getCalculateContentList(startDateStr: String, endDateStr: String): List<GetCalculateContentResponse> {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        return repository
            .getCalculateContentList(startDate, endDate)
            .map { it.toGetCalculateContentResponse() }
    }

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["cache_ttl_3_hours"],
        key = "'cumulativeSalesByContent:' + " + "#offset + ':' + #limit"
    )
    fun getCumulativeSalesByContent(offset: Long, limit: Long): GetCumulativeSalesByContentResponse {
        val totalCount = repository.getCumulativeSalesByContentTotalCount()
        val items = repository
            .getCumulativeSalesByContent(offset, limit)
            .map { it.toCumulativeSalesByContentItem() }

        return GetCumulativeSalesByContentResponse(totalCount, items)
    }

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["cache_ttl_3_hours"],
        key = "'calculateContentDonationList2:' + " + "#startDateStr + ':' + #endDateStr"
    )
    fun getCalculateContentDonationList(
        startDateStr: String,
        endDateStr: String
    ): List<GetCalculateContentDonationResponse> {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        return repository
            .getCalculateContentDonationList(startDate, endDate)
            .map { it.toGetCalculateContentDonationResponse() }
    }

    fun getCalculateCommunityPost(
        startDateStr: String,
        endDateStr: String,
        offset: Long,
        limit: Long
    ): GetCreatorCalculateCommunityPostResponse {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        val totalCount = repository.getCalculateCommunityPostTotalCount(startDate, endDate)
        val items = repository
            .getCalculateCommunityPostList(startDate, endDate, offset, limit)
            .map { it.toGetCalculateCommunityPostResponse() }

        return GetCreatorCalculateCommunityPostResponse(totalCount, items)
    }
}
