package kr.co.vividnext.sodalive.admin.calculate

import com.querydsl.core.annotations.QueryProjection
import kotlin.math.roundToInt

data class GetCalculateCommunityPostQueryData @QueryProjection constructor(
    val nickname: String,
    val title: String,
    val date: String,
    val can: Int,
    val numberOfPurchase: Long,
    val totalCan: Int,
    val settlementRatio: Int?
) {
    fun toGetCalculateCommunityPostResponse(): GetCalculateCommunityPostResponse {
        val totalKrw = totalCan * 100
        val paymentFee = totalKrw * 0.066f
        val settlementAmount = if (settlementRatio != null) {
            (totalKrw.toFloat() - paymentFee) * (settlementRatio.toFloat() / 100.0f)
        } else {
            (totalKrw.toFloat() - paymentFee) * 0.7f
        }
        val tax = settlementAmount * 0.033
        val depositAmount = settlementAmount - tax

        return GetCalculateCommunityPostResponse(
            nickname = nickname,
            title = title,
            date = date,
            can = can,
            numberOfPurchase = numberOfPurchase.toInt(),
            totalCan = totalCan,
            totalKrw = totalKrw,
            paymentFee = paymentFee.roundToInt(),
            settlementAmount = settlementAmount.roundToInt(),
            tax = tax.roundToInt(),
            depositAmount = depositAmount.roundToInt()
        )
    }
}
