package kr.co.vividnext.sodalive.admin.calculate

import com.querydsl.core.annotations.QueryProjection
import kotlin.math.roundToInt

data class GetCalculateContentDonationQueryData @QueryProjection constructor(
    // 등록 크리에이터 닉네임
    val nickname: String,
    // 콘텐츠 제목
    val title: String,
    // 콘텐츠 가격
    val price: Int,
    // 콘텐츠 등록 날짜
    val registrationDate: String,
    // 후원 날짜
    val donationDate: String,
    // 인원
    val numberOfDonation: Long,
    // 합계
    val totalCan: Int
) {
    fun toGetCalculateContentDonationResponse(): GetCalculateContentDonationResponse {
        // 원화 = totalCoin * 100 ( 캔 1개 = 100원 )
        val totalKrw = totalCan * 100

        // 결제수수료 : 6.6%
        val paymentFee = totalKrw * 0.066f

        // 정산금액
        // 유료콘텐츠 (원화 - 결제수수료) 의 90%
        // 무료콘텐츠 (원화 - 결제수수료) 의 70%
        val settlementAmount = if (price > 0) {
            (totalKrw.toFloat() - paymentFee) * 0.9
        } else {
            (totalKrw.toFloat() - paymentFee) * 0.7
        }

        // 원천세 = 정산금액의 3.3%
        val tax = settlementAmount * 0.033

        // 입금액
        val depositAmount = settlementAmount - tax

        val paidOrFree = if (price > 0) {
            "유료"
        } else {
            "무료"
        }

        return GetCalculateContentDonationResponse(
            nickname = nickname,
            title = title,
            paidOrFree = paidOrFree,
            registrationDate = registrationDate,
            donationDate = donationDate,
            numberOfDonation = numberOfDonation.toInt(),
            totalCan = totalCan,
            totalKrw = totalKrw,
            paymentFee = paymentFee.roundToInt(),
            settlementAmount = settlementAmount.roundToInt(),
            tax = tax.roundToInt(),
            depositAmount = depositAmount.roundToInt()
        )
    }
}
