package kr.co.vividnext.sodalive.admin.calculate

import com.fasterxml.jackson.annotation.JsonProperty
import com.querydsl.core.annotations.QueryProjection
import kr.co.vividnext.sodalive.content.order.OrderType
import kotlin.math.roundToInt

data class GetCumulativeSalesByContentQueryData @QueryProjection constructor(
    // 등록 크리에이터 닉네임
    val nickname: String,
    // 콘텐츠 제목
    val title: String,
    // 콘텐츠 등록 날짜
    val registrationDate: String,
    // 대여/소장 구분
    val orderType: OrderType,
    // 판매 금액(캔)
    val orderPrice: Int,
    // 인원
    val numberOfPeople: Long,
    // 합계
    val totalCan: Int
) {
    fun toCumulativeSalesByContentItem(): CumulativeSalesByContentItem {
        val orderTypeStr = if (orderType == OrderType.RENTAL) {
            "대여"
        } else {
            "소장"
        }

        // 원화 = totalCoin * 100 ( 캔 1개 = 100원 )
        val totalKrw = totalCan * 100

        // 결제수수료 : 6.6%
        val paymentFee = totalKrw * 0.066f

        // 정산금액 = (원화 - 결제수수료) 의 70%
        val settlementAmount = (totalKrw.toFloat() - paymentFee) * 0.7

        // 원천세 = 정산금액의 3.3%
        val tax = settlementAmount * 0.033

        // 입금액
        val depositAmount = settlementAmount - tax

        return CumulativeSalesByContentItem(
            nickname = nickname,
            title = title,
            registrationDate = registrationDate,
            orderType = orderTypeStr,
            orderPrice = orderPrice,
            numberOfPeople = numberOfPeople.toInt(),
            totalCan = totalCan,
            totalKrw = totalKrw,
            paymentFee = paymentFee.roundToInt(),
            settlementAmount = settlementAmount.roundToInt(),
            tax = tax.roundToInt(),
            depositAmount = depositAmount.roundToInt()
        )
    }
}

data class GetCumulativeSalesByContentResponse(
    @JsonProperty("totalCount") val totalCount: Int,
    @JsonProperty("items") val items: List<CumulativeSalesByContentItem>
)

data class CumulativeSalesByContentItem(
    @JsonProperty("nickname") val nickname: String,
    @JsonProperty("title") val title: String,
    @JsonProperty("registrationDate") val registrationDate: String,
    @JsonProperty("orderType") val orderType: String,
    @JsonProperty("orderPrice") val orderPrice: Int,
    @JsonProperty("numberOfPeople") val numberOfPeople: Int,
    @JsonProperty("totalCan") val totalCan: Int,
    @JsonProperty("totalKrw") val totalKrw: Int,
    @JsonProperty("paymentFee") val paymentFee: Int,
    @JsonProperty("settlementAmount") val settlementAmount: Int,
    @JsonProperty("tax") val tax: Int,
    @JsonProperty("depositAmount") val depositAmount: Int
)
