package kr.co.vividnext.sodalive.can.charge

import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.security.core.userdetails.User
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/charge")
class ChargeController(private val service: ChargeService) {

    @PostMapping
    fun charge(
        @RequestBody chargeRequest: ChargeRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.charge(member, chargeRequest))
    }

    @PostMapping("/verify")
    fun verify(
        @RequestBody verifyRequest: VerifyRequest,
        @AuthenticationPrincipal user: User
    ) = ApiResponse.ok(service.verify(user, verifyRequest))

    @PostMapping("/verify/hecto")
    fun verifyHecto(
        @RequestBody verifyRequest: VerifyRequest,
        @AuthenticationPrincipal user: User
    ) = ApiResponse.ok(service.verifyHecto(user, verifyRequest))

    @PostMapping("/apple")
    fun appleCharge(
        @RequestBody chargeRequest: AppleChargeRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.appleCharge(member, chargeRequest))
    }

    @PostMapping("/apple/verify")
    fun appleVerify(
        @RequestBody verifyRequest: AppleVerifyRequest,
        @AuthenticationPrincipal user: User
    ) = ApiResponse.ok(service.appleVerify(user, verifyRequest))

    @PostMapping("/google")
    fun googleCharge(
        @RequestBody request: GoogleChargeRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        if (request.paymentGateway == PaymentGateway.GOOGLE_IAP) {
            val chargeId = service.googleCharge(
                member = member,
                title = request.title,
                chargeCan = request.chargeCan,
                price = request.price,
                currencyCode = request.currencyCode,
                productId = request.productId,
                purchaseToken = request.purchaseToken,
                paymentGateway = request.paymentGateway
            )

            ApiResponse.ok(
                service.processGoogleIap(
                    memberId = member.id!!,
                    chargeId = chargeId,
                    productId = request.productId,
                    purchaseToken = request.purchaseToken,
                    paymentGateway = request.paymentGateway
                )
            )
        } else {
            throw SodaException("결제정보에 오류가 있습니다.")
        }
    }
}
