package kr.co.vividnext.sodalive.content.like

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.like.QAudioContentLike.audioContentLike
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface AudioContentLikeRepository : JpaRepository<AudioContentLike, Long>, AudioContentLikeQueryRepository

interface AudioContentLikeQueryRepository {
    fun findByMemberIdAndContentId(memberId: Long, contentId: Long): AudioContentLike?
    fun totalCountAudioContentLike(contentId: Long): Int
}

@Repository
class AudioContentLikeQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : AudioContentLikeQueryRepository {
    override fun findByMemberIdAndContentId(memberId: Long, contentId: Long): AudioContentLike? {
        return queryFactory
            .selectFrom(audioContentLike)
            .where(
                audioContentLike.memberId.eq(memberId)
                    .and(audioContentLike.audioContent.id.eq(contentId))
            )
            .fetchFirst()
    }

    override fun totalCountAudioContentLike(contentId: Long): Int {
        return queryFactory
            .select(audioContentLike.id)
            .from(audioContentLike)
            .where(
                audioContentLike.audioContent.id.eq(contentId)
                    .and(audioContentLike.isActive.isTrue)
            )
            .fetch()
            .size
    }
}
