package kr.co.vividnext.sodalive.content.series

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesSortType
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/audio-content/series")
class ContentSeriesController(private val service: ContentSeriesService) {
    @GetMapping
    fun getSeriesList(
        @RequestParam creatorId: Long,
        @RequestParam("sortType", required = false) sortType: SeriesSortType = SeriesSortType.NEWEST,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getSeriesList(
                creatorId = creatorId,
                sortType = sortType,
                member = member,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/{id}")
    fun getSeriesDetail(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getSeriesDetail(seriesId = id, member = member)
        )
    }

    @GetMapping("/{id}/content")
    fun getSeriesContentList(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getSeriesContentList(
                seriesId = id,
                member = member,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/recommend")
    fun getRecommendSeriesList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getRecommendSeriesList(member = member))
    }
}
