package kr.co.vividnext.sodalive.content.theme

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.SortType
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/audio-content/theme")
class AudioContentThemeController(private val service: AudioContentThemeService) {
    @GetMapping
    @PreAuthorize("hasRole('CREATOR')")
    fun getThemes(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getThemes())
    }

    @GetMapping("/{id}/content")
    fun getContentByTheme(
        @PathVariable id: Long,
        @RequestParam("sort-type", required = false) sortType: SortType = SortType.NEWEST,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getContentByTheme(
                themeId = id,
                sortType = sortType,
                member = member,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }
}
