package kr.co.vividnext.sodalive.explorer.profile.creatorCommunity

import com.querydsl.core.annotations.QueryProjection
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment.GetCommunityPostCommentListItem
import java.time.LocalDateTime

data class SelectCommunityPostResponse @QueryProjection constructor(
    val id: Long,
    val creatorId: Long,
    val creatorNickname: String,
    val creatorProfileUrl: String,
    val imagePath: String?,
    val content: String,
    val date: LocalDateTime,
    val isCommentAvailable: Boolean,
    val price: Int
) {
    fun toCommunityPostListResponse(
        imageHost: String,
        date: String,
        isLike: Boolean,
        memberId: Long,
        existOrdered: Boolean,
        likeCount: Int,
        commentCount: Int,
        firstComment: GetCommunityPostCommentListItem?
    ): GetCommunityPostListResponse {
        return GetCommunityPostListResponse(
            postId = id,
            creatorId = creatorId,
            creatorNickname = creatorNickname,
            creatorProfileUrl = "$imageHost/$creatorProfileUrl",
            imageUrl = if (imagePath != null) {
                "$imageHost/$imagePath"
            } else {
                null
            },
            content = if (price > 0 && memberId != creatorId) {
                if (existOrdered) {
                    content
                } else {
                    val length = content.codePointCount(0, content.length)
                    if (length > 15) {
                        val endIndex = content.offsetByCodePoints(0, 15)
                        content.substring(0, endIndex).plus("...")
                    } else {
                        val endIndex = content.offsetByCodePoints(0, length / 2)
                        content.substring(0, endIndex).plus("...")
                    }
                }
            } else {
                content
            },
            price = price,
            date = date,
            isCommentAvailable = isCommentAvailable,
            isAdult = false,
            isLike = isLike,
            existOrdered = existOrdered,
            likeCount = likeCount,
            commentCount = commentCount,
            firstComment = firstComment
        )
    }
}
