package kr.co.vividnext.sodalive.live.roulette.v2

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/v2/roulette")
class RouletteController(private val service: RouletteService) {
    @GetMapping("/creator")
    @PreAuthorize("hasRole('CREATOR')")
    fun getAllRoulette(
        @RequestParam creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getAllRoulette(creatorId = creatorId, memberId = member.id!!))
    }

    @PostMapping
    @PreAuthorize("hasRole('CREATOR')")
    fun createNewRoulette(
        @RequestBody request: CreateRouletteRequestV2,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null || member.role != MemberRole.CREATOR) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.createRoulette(memberId = member.id!!, request = request))
    }

    @PutMapping
    @PreAuthorize("hasRole('CREATOR')")
    fun updateNewRoulette(
        @RequestBody request: UpdateRouletteRequestV2,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null || member.role != MemberRole.CREATOR) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.updateRoulette(memberId = member.id!!, request = request))
    }

    @GetMapping
    fun getRoulette(
        @RequestParam creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getRoulette(creatorId = creatorId, memberId = member.id!!))
    }

    @PostMapping("/spin")
    fun spinRoulette(
        @RequestBody request: SpinRouletteRequestV2,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.spinRoulette(request = request, member = member))
    }

    @PostMapping("/refund/{id}")
    fun refundDonation(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.refundDonation(id, member))
    }
}
